package djmil.cordacheckers.states;

import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.annotations.CordaSerializable;

@CordaSerializable
public class Stone {
    
    @CordaSerializable
    public enum Type {
        MAN,
        KING;
    }

    @CordaSerializable
    public enum Color {
        WHITE,
        BLACK;

        public Color opposite() {
            switch (this) {
                case WHITE:
                    return BLACK;
                case BLACK:
                    return WHITE;
            }
            throw new RuntimeException("Unknown Color");
        }
    }

    private final Color color;
    private final Type type;

    public final static Stone WHITE_MAN  = new Stone(Stone.Color.WHITE, Stone.Type.MAN);
    public final static Stone WHITE_KING = new Stone(Stone.Color.WHITE, Stone.Type.KING);
    public final static Stone BLACK_MAN  = new Stone(Stone.Color.BLACK, Stone.Type.MAN);
    public final static Stone BLACK_KING = new Stone(Stone.Color.BLACK, Stone.Type.KING);

    @ConstructorForDeserialization
    public Stone(Color color, Type type) {
        this.color = color;
        this.type = type;
    }

    public Color getColor() {
        return color;
    }

    public Type getType() {
        return type;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Stone other = (Stone) obj;
        if (color != other.color)
            return false;
        if (type != other.type)
            return false;
        return true;
    }

}
