package djmil.cordacheckers.gameboard;

import java.time.Duration;
import java.time.Instant;
import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.contracts.GameCommand;
import djmil.cordacheckers.gameresult.GameResultBuilder;
import djmil.cordacheckers.gameresult.GameResultBuilder.Reason;
import djmil.cordacheckers.gamestate.CommitSubFlow;
import djmil.cordacheckers.gamestate.FlowResponce;
import djmil.cordacheckers.gamestate.GetFlow;
import djmil.cordacheckers.gamestate.View;
import djmil.cordacheckers.gamestate.ViewBuilder;
import djmil.cordacheckers.states.GameState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.FlowEngine;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.utxo.StateAndRef;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;

public class SurrenderFlow implements ClientStartableFlow{

     private final static Logger log = LoggerFactory.getLogger(SurrenderFlow.class);

     @CordaInject
     public JsonMarshallingService jsonMarshallingService;

     @CordaInject
     public UtxoLedgerService utxoLedgerService;

     @CordaInject
     public FlowEngine flowEngine;

     @Suspendable
     @Override
     public String call(ClientRequestBody requestBody) {
          SecureHash utxoTrxId = null;

          try {
               final GameCommand command = new GameCommand(GameCommand.Action.GAME_BOARD_SURRENDER);

               final UUID gameUuid = UUID.fromString(requestBody.getRequestBody());

               final StateAndRef<GameState> gameBoardSar = this.flowEngine
                    .subFlow(new GetFlow(gameUuid));

               final GameResultBuilder.Result out = this.flowEngine
                    .subFlow(new GameResultBuilder(gameBoardSar, Reason.SURRENDER));

               final UtxoSignedTransaction gameBoardSurrenderTrx = utxoLedgerService.createTransactionBuilder()
                    .addCommand(command)
                    .addInputState(gameBoardSar.getRef())
                    .addOutputState(out.gameResult)
                    .addSignatories(out.gameResult.getParticipants())
                    .setNotary(gameBoardSar.getState().getNotaryName())
                    .setTimeWindowUntil(Instant.now().plusMillis(Duration.ofDays(1).toMillis()))
                    .toSignedTransaction();

               utxoTrxId = this.flowEngine
                    .subFlow(new CommitSubFlow(gameBoardSurrenderTrx, 
                         command.getCounterparty(gameBoardSar), 
                         out.custodyName));

               final View gameStateView = this.flowEngine
                    .subFlow(new ViewBuilder(utxoTrxId));

               return new FlowResponce(gameStateView, utxoTrxId)
                    .toJsonEncodedString(jsonMarshallingService);
          } 
          catch (Exception e) {
               log.warn("Exception during processing " + requestBody + " request: " + e.getMessage());
               e.printStackTrace(System.out);
               return new FlowResponce(e, utxoTrxId)
                    .toJsonEncodedString(jsonMarshallingService);
          }
     }

}
