package djmil.cordacheckers.gameresult;

import djmil.cordacheckers.states.GameBoardState;
import djmil.cordacheckers.states.GameResultState;
import djmil.cordacheckers.states.GameState;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.SubFlow;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.StateAndRef;
import net.corda.v5.membership.MemberInfo;

public class GameResultBuilder implements SubFlow<GameResultBuilder.Result> {
    public class Result {
        final public GameResultState gameResult;
        final public MemberX500Name custodyName;
      
        public Result(GameResultState gameResult, MemberX500Name custodyName){
            this.gameResult = gameResult;
            this.custodyName = custodyName;
        }
    }

    public static enum Reason {
        VICTORY,
        SURRENDER
    }

    private final GameBoardState gameBoard;
    private final Reason reason;

    public GameResultBuilder(StateAndRef<GameState> gameBoardSar, Reason reason) {
        this.gameBoard = (GameBoardState)gameBoardSar.getState().getContractState();
        this.reason = reason;
    }

    @CordaInject
    public MemberLookup memberLookup;

    @Override
    @Suspendable
    public Result call() {
        final MemberInfo custodiaInfo = findCustodian();
         
        return new Result(
            new GameResultState(gameBoard, winnerName(), custodiaInfo.getLedgerKeys().get(0)),
            custodiaInfo.getName());
    }

    @Suspendable
    MemberInfo findCustodian() {
        return memberLookup.lookup()
            .stream()
            .filter(m -> m.getName().getOrganizationUnit().equals("Custodian") )
            .reduce((a,b) -> {throw new IllegalStateException("Multiple Custodian VNodes");})
            .orElseThrow( () -> new IllegalStateException("No Custodian VNode found"));
    }

    @Suspendable 
    MemberX500Name winnerName() {
        final MemberX500Name myName = memberLookup.myInfo().getName();
        
        switch(reason) {
            case VICTORY:
                return myName;
            case SURRENDER:
                return gameBoard.getOpponentName(myName);
        } 

        throw new IllegalStateException("Bad reason");
    }

}
