package djmil.cordacheckers.gameresult;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.states.GameResultState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.FlowEngine;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.ledger.utxo.UtxoLedgerService;

public class RankingFlow implements ClientStartableFlow {

    private final static Logger log = LoggerFactory.getLogger(RankingFlow.class);

    @CordaInject
    public UtxoLedgerService utxoLedgerService;

    @CordaInject
    public JsonMarshallingService jsonMarshallingService;

    @CordaInject
    public FlowEngine flowEngine;

    @Suspendable
    @Override
    public String call(ClientRequestBody requestBody) {
        try {
            final List<GameResultState> gameStateResutList = getGameResultsList();

            Map<String, Integer> won = new HashMap<>();
            for (GameResultState gs : gameStateResutList) {
                final var winner = gs.getWinnerName().getCommonName();
                won.put(winner, won.getOrDefault(winner, 0) +1);
            }

            Map<String, Integer> played = new HashMap<>(won);
            for (GameResultState gs : gameStateResutList) {
                final var looseer = gs.getLooserName().getCommonName();
                played.put(looseer, played.getOrDefault(looseer, 0) +1);
            }

            List<Rank> board = new LinkedList<Rank>();
            for (String name : played.keySet()) {
                board.add(new Rank(name, played.getOrDefault(name, 0), won.getOrDefault(name, 0)));
            }
    
            return new RankingFlowResponce(board)
                .toJsonEncodedString(jsonMarshallingService);
        } catch (Exception e) {
            log.warn("Exception during processing " + requestBody + " request: " + e.getMessage());
            return new RankingFlowResponce(e)
                .toJsonEncodedString(jsonMarshallingService);
        }
    }

    @Suspendable
    List<GameResultState> getGameResultsList() {
        return utxoLedgerService
            .findUnconsumedStatesByType(GameResultState.class)
            .stream()
            .map( stateAndRef -> stateAndRef.getState().getContractState() )
            .collect(Collectors.toList());
    }

}
